@@ -1,6 +1,6 @@
 ## fastLm.R: Rcpp/Armadillo implementation of lm()
 ##
-## Copyright (C)  2010 - 2017  Dirk Eddelbuettel, Romain Francois and Douglas Bates
+## Copyright (C)  2010 - 2021  Dirk Eddelbuettel, Romain Francois and Douglas Bates
 ##
 ## This file is part of RcppArmadillo.
 ##
@@ -128,7 +128,7 @@ predict.fastLm <- function(object, newdata=NULL, ...) {
         if (!is.null(object$formula)) {
             x <- model.matrix(object$formula, newdata)
         } else {
-            x <- newdata
+            x <- newdata 						# #nocov
         }
         y <- as.vector(x %*% coef(object))
     }
