@@ -1,9 +1,7 @@
-// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-
-/* :tabSize=4:indentSize=4:noTabs=false:folding=explicit:collapseFolds=1: */
-//
+
 // RcppArmadillo.h: Rcpp/Armadillo glue
 //
-// Copyright (C)  2010 - 2014  Dirk Eddelbuettel, Romain Francois and Douglas Bates
+// Copyright (C)  2010 - 2021  Dirk Eddelbuettel, Romain Francois and Douglas Bates
 //
 // This file is part of RcppArmadillo.
 //
@@ -27,11 +25,15 @@
     #error "The file 'Rcpp.h' should not be included. Please correct to include only 'RcppArmadillo.h'."
 #endif
 
-#include <RcppArmadilloForward.h>
+// Set up actual #include <armadillo> after first #include <RcppArmadilloConfig> and more config
+#include <RcppArmadillo/interface/RcppArmadilloForward.h>
+
+// Now automatically include Rcpp as well
 #include <Rcpp.h>
-#include <RcppArmadilloWrap.h>
-#include <RcppArmadilloAs.h>
-#include <RcppArmadilloSugar.h>
+
+// Remaining RcppArmadillo code
+#include <RcppArmadillo/interface/RcppArmadilloWrap.h>
+#include <RcppArmadillo/interface/RcppArmadilloAs.h>
+#include <RcppArmadillo/interface/RcppArmadilloSugar.h>
 
 #endif
-
