@@ -22,8 +22,8 @@
 
 
 template<typename T1>
-inline
 arma_warn_unused
+inline
 typename enable_if2< is_arma_type<T1>::value, typename T1::pod_type >::result
 norm
   (
@@ -48,7 +48,7 @@ norm
     if(k == uword(1))  { return op_norm::vec_norm_1(P); }
     if(k == uword(2))  { return op_norm::vec_norm_2(P); }
     
-    arma_debug_check( (k == 0), "norm(): k must be greater than zero" );
+    arma_debug_check( (k == 0), "norm(): unsupported vector norm type" );
     
     return op_norm::vec_norm_k(P, int(k));
     }
@@ -68,8 +68,8 @@ norm
 
 
 template<typename T1>
-inline
 arma_warn_unused
+inline
 typename enable_if2< is_arma_type<T1>::value, typename T1::pod_type >::result
 norm
   (
@@ -120,13 +120,57 @@ norm
 
 
 
+template<typename T1>
+arma_warn_unused
+inline
+typename enable_if2< is_arma_type<T1>::value, double >::result
+norm
+  (
+  const T1&   X,
+  const uword k = uword(2),
+  const typename arma_integral_only<typename T1::elem_type>::result* junk = nullptr
+  )
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  if(resolves_to_colvector<T1>::value)  { return norm(conv_to< Col<double> >::from(X), k); }
+  if(resolves_to_rowvector<T1>::value)  { return norm(conv_to< Row<double> >::from(X), k); }
+  
+  return norm(conv_to< Mat<double> >::from(X), k);
+  }
+
+
+
+template<typename T1>
+arma_warn_unused
+inline
+typename enable_if2< is_arma_type<T1>::value, double >::result
+norm
+  (
+  const T1&   X,
+  const char* method,
+  const typename arma_integral_only<typename T1::elem_type>::result* junk = nullptr
+  )
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  if(resolves_to_colvector<T1>::value)  { return norm(conv_to< Col<double> >::from(X), method); }
+  if(resolves_to_rowvector<T1>::value)  { return norm(conv_to< Row<double> >::from(X), method); }
+  
+  return norm(conv_to< Mat<double> >::from(X), method);
+  }
+
+
+
 //
 // norms for sparse matrices
 
 
 template<typename T1>
-inline
 arma_warn_unused
+inline
 typename enable_if2< is_arma_sparse_type<T1>::value, typename T1::pod_type >::result
 norm
   (
@@ -141,6 +185,20 @@ norm
   typedef typename T1::elem_type eT;
   typedef typename T1::pod_type   T;
   
+  if(is_SpSubview_col<T1>::value)
+    {
+    const SpSubview_col<eT>& sv = reinterpret_cast< const SpSubview_col<eT>& >(expr);
+    
+    if(sv.n_rows == sv.m.n_rows)
+      {
+      const SpMat<eT>& m   = sv.m;
+      const uword      col = sv.aux_col1;
+      const eT*        mem = &(m.values[ m.col_ptrs[col] ]);
+      
+      return spop_norm::vec_norm_k(mem, sv.n_nonzero, k);
+      }
+    }
+  
   const unwrap_spmat<T1> U(expr);
   const SpMat<eT>& X   = U.M;
   
@@ -150,17 +208,7 @@ norm
   
   if(is_vec)
     {
-    // create a fake dense vector to allow reuse of code for dense vectors
-    Col<eT> fake_vector( access::rwp(X.values), X.n_nonzero, false );
-    
-    const Proxy< Col<eT> > P_fake_vector(fake_vector);
-    
-    if(k == uword(1))  { return op_norm::vec_norm_1(P_fake_vector); }
-    if(k == uword(2))  { return op_norm::vec_norm_2(P_fake_vector); }
-    
-    arma_debug_check( (k == 0), "norm(): k must be greater than zero" );
-    
-    return op_norm::vec_norm_k(P_fake_vector, int(k));
+    return spop_norm::vec_norm_k(X.values, X.n_nonzero, k);
     }
   else
     {
@@ -176,8 +224,8 @@ norm
 
 
 template<typename T1>
-inline
 arma_warn_unused
+inline
 typename enable_if2< is_arma_sparse_type<T1>::value, typename T1::pod_type >::result
 norm
   (
@@ -247,4 +295,48 @@ norm
 
 
 
+//
+// approximate norms
+
+
+template<typename T1>
+arma_warn_unused
+inline
+typename T1::pod_type
+norm2est
+  (
+  const Base<typename T1::elem_type, T1>& X,
+  const typename T1::pod_type             tolerance = 0,
+  const uword                             max_iter  = 100,
+  const typename arma_real_or_cx_only<typename T1::elem_type>::result* junk = nullptr
+  )
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  return op_norm2est::norm2est(X.get_ref(), tolerance, max_iter);
+  }
+
+
+
+template<typename T1>
+arma_warn_unused
+inline
+typename T1::pod_type
+norm2est
+  (
+  const SpBase<typename T1::elem_type, T1>& X,
+  const typename T1::pod_type               tolerance = 0,
+  const uword                               max_iter  = 100,
+  const typename arma_real_or_cx_only<typename T1::elem_type>::result* junk = nullptr
+  )
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  return op_norm2est::norm2est(X.get_ref(), tolerance, max_iter);
+  }
+
+
+
 //! @}
