@@ -1,12 +1,19 @@
-// Copyright (C) 2010-2016 National ICT Australia (NICTA)
+// SPDX-License-Identifier: Apache-2.0
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
-// Written by Dimitrios Bouzas
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
+// 
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 
@@ -37,7 +44,7 @@ op_find::helper
   uword* indices_mem = indices.memptr();
   uword  n_nz        = 0;
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     typename Proxy<T1>::ea_type PA = A.get_ea();
     
@@ -74,8 +81,8 @@ op_find::helper
   (
   Mat<uword>& indices,
   const mtOp<uword, T1, op_type>& X,
-  const typename arma_op_rel_only<op_type>::result           junk1,
-  const typename arma_not_cx<typename T1::elem_type>::result junk2
+  const typename arma_op_rel_only<op_type>::result*           junk1,
+  const typename arma_not_cx<typename T1::elem_type>::result* junk2
   )
   {
   arma_extra_debug_sigprint();
@@ -86,6 +93,11 @@ op_find::helper
   
   const eT val = X.aux;
   
+  if((is_same_type<op_type, op_rel_eq>::yes || is_same_type<op_type, op_rel_noteq>::yes) && arma_config::debug && arma_isnan(val))
+    {
+    arma_debug_warn_level(1, "find(): NaN is not equal to anything; suggest to use find_nonfinite() instead");
+    }
+  
   const Proxy<T1> A(X.m);
   
   const uword n_elem = A.get_n_elem();
@@ -95,7 +107,7 @@ op_find::helper
   uword* indices_mem = indices.memptr();
   uword  n_nz        = 0;
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     typename Proxy<T1>::ea_type PA = A.get_ea();
     
@@ -132,8 +144,8 @@ op_find::helper
       else if(is_same_type<op_type, op_rel_noteq    >::yes)  { not_zero_j = (tpj != val); }
       else { not_zero_j = false; }
       
-      if(not_zero_i == true)  { indices_mem[n_nz] = i;  ++n_nz; }
-      if(not_zero_j == true)  { indices_mem[n_nz] = j;  ++n_nz; }
+      if(not_zero_i)  { indices_mem[n_nz] = i;  ++n_nz; }
+      if(not_zero_j)  { indices_mem[n_nz] = j;  ++n_nz; }
       }
     
     if(i < n_elem)
@@ -154,7 +166,7 @@ op_find::helper
       else if(is_same_type<op_type, op_rel_noteq    >::yes)  { not_zero = (tmp != val); }
       else { not_zero = false; }
       
-      if(not_zero == true)  { indices_mem[n_nz] = i;  ++n_nz; }
+      if(not_zero)  { indices_mem[n_nz] = i;  ++n_nz; }
       }
     }
   else
@@ -183,7 +195,7 @@ op_find::helper
       else if(is_same_type<op_type, op_rel_noteq    >::yes)  { not_zero = (tmp != val); }
       else { not_zero = false; }
       
-      if(not_zero == true)  { indices_mem[n_nz] = i;  ++n_nz; }
+      if(not_zero)  { indices_mem[n_nz] = i;  ++n_nz; }
       
       ++i;
       }
@@ -201,8 +213,8 @@ op_find::helper
   (
   Mat<uword>& indices,
   const mtOp<uword, T1, op_type>& X,
-  const typename arma_op_rel_only<op_type>::result            junk1,
-  const typename arma_cx_only<typename T1::elem_type>::result junk2
+  const typename arma_op_rel_only<op_type>::result*            junk1,
+  const typename arma_cx_only<typename T1::elem_type>::result* junk2
   )
   {
   arma_extra_debug_sigprint();
@@ -214,9 +226,13 @@ op_find::helper
   
   const eT val = X.aux;
   
+  if((is_same_type<op_type, op_rel_eq>::yes || is_same_type<op_type, op_rel_noteq>::yes) && arma_config::debug && arma_isnan(val))
+    {
+    arma_debug_warn_level(1, "find(): NaN is not equal to anything; suggest to use find_nonfinite() instead");
+    }
+  
   const Proxy<T1> A(X.m);
   
-  ea_type     PA     = A.get_ea();
   const uword n_elem = A.get_n_elem();
   
   indices.set_size(n_elem, 1);
@@ -225,8 +241,10 @@ op_find::helper
   uword  n_nz        = 0;
   
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
+    ea_type PA = A.get_ea();
+    
     for(uword i=0; i<n_elem; ++i)
       {
       const eT tmp = PA[i];
@@ -237,7 +255,7 @@ op_find::helper
       else if(is_same_type<op_type, op_rel_noteq>::yes)  { not_zero = (tmp != val); }
       else { not_zero = false; }
       
-      if(not_zero == true) { indices_mem[n_nz] = i;  ++n_nz; }
+      if(not_zero) { indices_mem[n_nz] = i;  ++n_nz; }
       }
     }
   else
@@ -258,7 +276,7 @@ op_find::helper
       else if(is_same_type<op_type, op_rel_noteq>::yes)  { not_zero = (tmp != val); }
       else { not_zero = false; }
       
-      if(not_zero == true) { indices_mem[n_nz] = i;  ++n_nz; }
+      if(not_zero) { indices_mem[n_nz] = i;  ++n_nz; }
       
       i++;
       }
@@ -276,9 +294,9 @@ op_find::helper
   (
   Mat<uword>& indices,
   const mtGlue<uword, T1, T2, glue_type>& X,
-  const typename arma_glue_rel_only<glue_type>::result       junk1,
-  const typename arma_not_cx<typename T1::elem_type>::result junk2,
-  const typename arma_not_cx<typename T2::elem_type>::result junk3
+  const typename arma_glue_rel_only<glue_type>::result*       junk1,
+  const typename arma_not_cx<typename T1::elem_type>::result* junk2,
+  const typename arma_not_cx<typename T2::elem_type>::result* junk3
   )
   {
   arma_extra_debug_sigprint();
@@ -297,34 +315,67 @@ op_find::helper
   
   arma_debug_assert_same_size(A, B, "relational operator");
   
-  ea_type1 PA = A.get_ea();
-  ea_type2 PB = B.get_ea();
-  
-  const uword n_elem = B.get_n_elem();
+  const uword n_elem = A.get_n_elem();
   
   indices.set_size(n_elem, 1);
   
   uword* indices_mem = indices.memptr();
   uword  n_nz        = 0;
   
-  for(uword i=0; i<n_elem; ++i)
+  if((Proxy<T1>::use_at == false) && (Proxy<T2>::use_at == false))
     {
-    const eT1 tmp1 = PA[i];
-    const eT2 tmp2 = PB[i];
+    ea_type1 PA = A.get_ea();
+    ea_type2 PB = B.get_ea();
     
-    bool not_zero;
+    for(uword i=0; i<n_elem; ++i)
+      {
+      const eT1 tmp1 = PA[i];
+      const eT2 tmp2 = PB[i];
+      
+      bool not_zero;
+      
+           if(is_same_type<glue_type, glue_rel_lt    >::yes)  { not_zero = (tmp1 <  tmp2); }
+      else if(is_same_type<glue_type, glue_rel_gt    >::yes)  { not_zero = (tmp1 >  tmp2); }
+      else if(is_same_type<glue_type, glue_rel_lteq  >::yes)  { not_zero = (tmp1 <= tmp2); }
+      else if(is_same_type<glue_type, glue_rel_gteq  >::yes)  { not_zero = (tmp1 >= tmp2); }
+      else if(is_same_type<glue_type, glue_rel_eq    >::yes)  { not_zero = (tmp1 == tmp2); }
+      else if(is_same_type<glue_type, glue_rel_noteq >::yes)  { not_zero = (tmp1 != tmp2); }
+      else if(is_same_type<glue_type, glue_rel_and   >::yes)  { not_zero = (tmp1 && tmp2); }
+      else if(is_same_type<glue_type, glue_rel_or    >::yes)  { not_zero = (tmp1 || tmp2); }
+      else { not_zero = false; }
+      
+      if(not_zero)  { indices_mem[n_nz] = i;  ++n_nz; }
+      }
+    }
+  else
+    {
+    const uword n_rows = A.get_n_rows();
+    const uword n_cols = A.get_n_cols();
     
-         if(is_same_type<glue_type, glue_rel_lt    >::yes)  { not_zero = (tmp1 <  tmp2); }
-    else if(is_same_type<glue_type, glue_rel_gt    >::yes)  { not_zero = (tmp1 >  tmp2); }
-    else if(is_same_type<glue_type, glue_rel_lteq  >::yes)  { not_zero = (tmp1 <= tmp2); }
-    else if(is_same_type<glue_type, glue_rel_gteq  >::yes)  { not_zero = (tmp1 >= tmp2); }
-    else if(is_same_type<glue_type, glue_rel_eq    >::yes)  { not_zero = (tmp1 == tmp2); }
-    else if(is_same_type<glue_type, glue_rel_noteq >::yes)  { not_zero = (tmp1 != tmp2); }
-    else if(is_same_type<glue_type, glue_rel_and   >::yes)  { not_zero = (tmp1 && tmp2); }
-    else if(is_same_type<glue_type, glue_rel_or    >::yes)  { not_zero = (tmp1 || tmp2); }
-    else { not_zero = false; }
+    uword i = 0;
     
-    if(not_zero == true)  { indices_mem[n_nz] = i;  ++n_nz; }
+    for(uword col=0; col < n_cols; ++col)
+    for(uword row=0; row < n_rows; ++row)
+      {
+      const eT1 tmp1 = A.at(row,col);
+      const eT2 tmp2 = B.at(row,col);
+      
+      bool not_zero;
+      
+           if(is_same_type<glue_type, glue_rel_lt    >::yes)  { not_zero = (tmp1 <  tmp2); }
+      else if(is_same_type<glue_type, glue_rel_gt    >::yes)  { not_zero = (tmp1 >  tmp2); }
+      else if(is_same_type<glue_type, glue_rel_lteq  >::yes)  { not_zero = (tmp1 <= tmp2); }
+      else if(is_same_type<glue_type, glue_rel_gteq  >::yes)  { not_zero = (tmp1 >= tmp2); }
+      else if(is_same_type<glue_type, glue_rel_eq    >::yes)  { not_zero = (tmp1 == tmp2); }
+      else if(is_same_type<glue_type, glue_rel_noteq >::yes)  { not_zero = (tmp1 != tmp2); }
+      else if(is_same_type<glue_type, glue_rel_and   >::yes)  { not_zero = (tmp1 && tmp2); }
+      else if(is_same_type<glue_type, glue_rel_or    >::yes)  { not_zero = (tmp1 || tmp2); }
+      else { not_zero = false; }
+      
+      if(not_zero)  { indices_mem[n_nz] = i;  ++n_nz; }
+      
+      i++;
+      }
     }
   
   return n_nz;
@@ -339,9 +390,9 @@ op_find::helper
   (
   Mat<uword>& indices,
   const mtGlue<uword, T1, T2, glue_type>& X,
-  const typename arma_glue_rel_only<glue_type>::result        junk1,
-  const typename arma_cx_only<typename T1::elem_type>::result junk2,
-  const typename arma_cx_only<typename T2::elem_type>::result junk3
+  const typename arma_glue_rel_only<glue_type>::result*        junk1,
+  const typename arma_cx_only<typename T1::elem_type>::result* junk2,
+  const typename arma_cx_only<typename T2::elem_type>::result* junk3
   )
   {
   arma_extra_debug_sigprint();
@@ -357,19 +408,18 @@ op_find::helper
   
   arma_debug_assert_same_size(A, B, "relational operator");
   
-  ea_type1 PA = A.get_ea();
-  ea_type2 PB = B.get_ea();
-  
-  const uword n_elem = B.get_n_elem();
+  const uword n_elem = A.get_n_elem();
   
   indices.set_size(n_elem, 1);
   
   uword* indices_mem = indices.memptr();
   uword  n_nz        = 0;
   
-  
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if((Proxy<T1>::use_at == false) && (Proxy<T2>::use_at == false))
     {
+    ea_type1 PA = A.get_ea();
+    ea_type2 PB = B.get_ea();
+    
     for(uword i=0; i<n_elem; ++i)
       {
       bool not_zero;
@@ -378,7 +428,7 @@ op_find::helper
       else if(is_same_type<glue_type, glue_rel_noteq >::yes)  { not_zero = (PA[i] != PB[i]); }
       else { not_zero = false; }
       
-      if(not_zero == true)  { indices_mem[n_nz] = i;  ++n_nz; }
+      if(not_zero)  { indices_mem[n_nz] = i;  ++n_nz; }
       }
     }
   else
@@ -397,11 +447,11 @@ op_find::helper
       else if(is_same_type<glue_type, glue_rel_noteq >::yes)  { not_zero = (A.at(row,col) != B.at(row,col)); }
       else { not_zero = false; }
       
-      if(not_zero == true)  { indices_mem[n_nz] = i;  ++n_nz; }
+      if(not_zero)  { indices_mem[n_nz] = i;  ++n_nz; }
       
       i++;
       }
-   }
+    }
   
   return n_nz;
   }
@@ -470,16 +520,18 @@ op_find_finite::apply(Mat<uword>& out, const mtOp<uword, T1, op_find_finite>& X)
   {
   arma_extra_debug_sigprint();
   
+  if(arma_config::fast_math)  { arma_debug_warn_level(2, "find_finite(): detection of non-finite values is not reliable in fast math mode"); }
+  
   const Proxy<T1> P(X.m);
   
   const uword n_elem = P.get_n_elem();
   
-  Mat<uword> indices(n_elem,1);
+  Mat<uword> indices(n_elem, 1, arma_nozeros_indicator());
   
   uword* indices_mem = indices.memptr();
   uword  count       = 0;
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     const typename Proxy<T1>::ea_type Pea = P.get_ea();
     
@@ -516,16 +568,18 @@ op_find_nonfinite::apply(Mat<uword>& out, const mtOp<uword, T1, op_find_nonfinit
   {
   arma_extra_debug_sigprint();
   
+  if(arma_config::fast_math)  { arma_debug_warn_level(2, "find_nonfinite(): detection of non-finite values is not reliable in fast math mode"); }
+  
   const Proxy<T1> P(X.m);
   
   const uword n_elem = P.get_n_elem();
   
-  Mat<uword> indices(n_elem,1);
+  Mat<uword> indices(n_elem, 1, arma_nozeros_indicator());
   
   uword* indices_mem = indices.memptr();
   uword  count       = 0;
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     const typename Proxy<T1>::ea_type Pea = P.get_ea();
     
@@ -555,4 +609,52 @@ op_find_nonfinite::apply(Mat<uword>& out, const mtOp<uword, T1, op_find_nonfinit
 
 
 
+template<typename T1>
+inline
+void
+op_find_nan::apply(Mat<uword>& out, const mtOp<uword, T1, op_find_nan>& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  if(arma_config::fast_math)  { arma_debug_warn_level(2, "find_nan(): detection of non-finite values is not reliable in fast math mode"); }
+  
+  const Proxy<T1> P(X.m);
+  
+  const uword n_elem = P.get_n_elem();
+  
+  Mat<uword> indices(n_elem, 1, arma_nozeros_indicator());
+  
+  uword* indices_mem = indices.memptr();
+  uword  count       = 0;
+  
+  if(Proxy<T1>::use_at == false)
+    {
+    const typename Proxy<T1>::ea_type Pea = P.get_ea();
+    
+    for(uword i=0; i<n_elem; ++i)
+      {
+      if( arma_isnan(Pea[i]) )  { indices_mem[count] = i; count++; }
+      }
+    }
+  else
+    {
+    const uword n_rows = P.get_n_rows(); 
+    const uword n_cols = P.get_n_cols(); 
+    
+    uword i = 0;
+    
+    for(uword col=0; col<n_cols; ++col)
+    for(uword row=0; row<n_rows; ++row)
+      {
+      if( arma_isnan(P.at(row,col)) )  { indices_mem[count] = i; count++; }
+      
+      i++;
+      }
+    }
+  
+  out.steal_mem_col(indices, count);
+  }
+
+
+
 //! @}
